
// Copyright (c) 2018, Tobias Mueller tm(at)tm3d.de
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the
//    distribution.
//  * All advertising materials mentioning features or use of this
//    software must display the following acknowledgement: This product
//    includes software developed by tm3d.de and its contributors.
//  * Neither the name of tm3d.de nor the names of its contributors may
//    be used to endorse or promote products derived from this software
//    without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

//#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
//#include "../common/I2C/TWI_Master.h"
#include "../common/owSlave_tools.h"
//#include "../common/I2C/SHT3x.h"
//#include "../common/calibr.h"


#include <math.h>

extern void OWINIT();
extern void EXTERN_SLEEP();
extern uint8_t stat_to_sample;

uint8_t owid[8]={0x29, 0x0C, 0xE5, 0x84, 0x00, 0x00, 0x03, 0x63}; /*6303000084E50C29*/
uint8_t config_info[26]={0x06,0x09,0x06,0x09,0x06,0x09,0x06,0x09,0x02,20,20,20,20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

OWST_EXTERN_VARS

OWST_WDT_ISR

typedef union {
	volatile uint8_t bytes[0x20];
	struct {
		uint8_t PIO_Logic_State;     //		0088h
		uint8_t PIO_Output_Latch_State;
		uint8_t PIO_Activity_Latch_State;
		uint8_t Conditional_Search_Channel_Selection_Mask;
		uint8_t Conditional_Search_Channel_Polarity_Selection;
		uint8_t Status; //008D
		uint8_t FF1;
		uint8_t FF2;
		
	};
	struct {
		uint8_t fill[6];     //		0088h
		uint16_t FF16;
	};
} pack_t;
volatile pack_t pack;

//uint8_t crc8() {
//	uint8_t lscrc=0x0;
//	for(uint8_t i=0;i<5;i++) {
//		uint8_t v=values[i];
//		//if (v==0) v=0xFF;
//		uint8_t bit=1;
//		uint8_t lb;
//		for(uint8_t j=0;j<8;j++) {
//			if ((v&bit)==bit) lb=1; else lb=0;
//			if ((lscrc&1)!=lb)	lscrc=(lscrc>>1)^0x8c; else	lscrc=(lscrc>>1);
//			bit=bit*2;
//		}
//	}
//	return lscrc;
//}

uint16_t ADmess() {
	 ADMUX=0b00101100;  //3V  ADC2+  ADC1- 1x
	 ADCSRA|=(1<<ADSC);
	 while ((ADCSRA&(1<<ADSC)));
	return ADC;
}

#define DDRA_OUT_MASK 0b11110000

int main(void){
	uint16_t adv_tmp;
	static uint16_t adv;
	static uint8_t pa_last;
	uint8_t pa_now;

	OWST_INIT_ADC_ON;

	ADMUX = 0; // ref vcc in PA0
	ADCSRA = 0x87; // Enable ADC  prescaler 128
	ADCSRB = 0; // unipolar right adjust free running
	// start conversion
	ADCSRA|=(1<<ADSC);

	DDRA=DDRA_OUT_MASK;
	PORTA=0b00001100;

	// switch als eingang (PB1)
	DDRB&=0b11111101;
	// pulup on (PB1)
	PORTB|=0b00000010;

	pack.FF1=0xFF;
	pack.FF2=0xFF;
	OWINIT();

	OWST_WDR_CONFIG8;
	sei();

	stat_to_sample=PORTA;
	pack.PIO_Output_Latch_State=PORTA;
	pack.PIO_Activity_Latch_State=0;
	pa_now=PINA&0xFE;
	if(PINB&0x02)
	{
		pa_now|=1;
	}
	pa_last=pa_now;

	while ((ADCSRA&(1<<ADSC)));
	adv=ADC<<4;

	while (1) {
		pa_now=PINA&0xFE;
		if(PINB&0x02)
		{
			pa_now|=1;
		}
		//stat_to_sample=0;
		if (reset_indicator) {
		//	ap=0;
		//	stat_to_sample=0;
		//	reset_indicator=0;
		}
		if (wdcounter>3) {
			
			wdcounter=0;
		}
		pack.Status|=0x80;
		if (gcontrol&1) 
		{
			// write ok 0xaa write channel
			// collect latch

			uint8_t tpa=PORTA;
			tpa&=~DDRA_OUT_MASK;
			tpa|=pack.PIO_Output_Latch_State&DDRA_OUT_MASK;
			PORTA=tpa;
			gcontrol&=~0x01;
		}
		if (gcontrol&2) 
		{
			// reset_activity
			pack.PIO_Activity_Latch_State=0;
			pa_last=pa_now;
            gcontrol&=~0x02;
		}
		if (gcontrol&4) 
		{
			// read channel
			stat_to_sample=pa_now;
			pack.PIO_Logic_State=stat_to_sample;
			gcontrol&=~0x04;
		} 		
		if (gcontrol&8) 
		{
			// read channel
			stat_to_sample=pa_now;
			pack.PIO_Logic_State=stat_to_sample;
			gcontrol&=~0x08;
		}
		// test ADC ready
		if(!(ADCSRA&(1<<ADSC)))
		{
			adv_tmp=adv>>4;
			adv-=adv_tmp;
			adv+=ADC;
			// store the ADC Value 
			pack.FF16=adv_tmp;
			// restart conversion
			ADCSRA|=(1<<ADSC);
		}
		pack.PIO_Logic_State=pa_now;
		pack.PIO_Output_Latch_State=PORTA;
		if(pa_last!=pa_now)
		{
			pack.PIO_Activity_Latch_State|=(pa_now^pa_last);
			pa_last=pa_now;
		}
		OWST_MAIN_END
	}
}
